---
id: 587d825c367417b2b2512c8f
title: Реалізація сортування злиттям
challengeType: 1
forumTopicId: 301614
dashedName: implement-merge-sort
---

# --description--

Ще один поширений алгоритм проміжного сортування - сортування злиттям. Як і швидке сортування, сортування злиттям також використовує метод розділяй і володарюй, рекурсивну методологію для сортування масиву. Він використовує перевагу в тому, що два масиви порівняно легко відсортувати, доки кожен з них відсортовано в першу чергу. Але ми почнемо тільки з одного масиву як з вхідного, то як же отримати з нього два відсортованих масиви? Ми можемо рекурсивно поділити початковий запис два, доки ми не досягнемо базового випадку масиву з одним предметом. Одноелементний масив природно відсортований, тому ми можемо почати комбінувати. Ця комбінація розгорне рекурсивні виклики, які розбивають оригінальний масив, врешті створюючи остаточний сортований масив усіх елементів. Кроки сортування злиттям:

**1)** Рекурсивно розділити вхідний масив навпіл, поки підмасив лише з одним елементом не буде сформовано.

**2)** Об'єднати кожен сортований підмасив разом для створення остаточного сортованого масиву.

Сортування злиттям є ефективним методом сортування зі складністю часу *O(nlog(n))*. Цей алгоритм популярний, оскільки він є продуктивним та відносно простим для реалізації.

До речі, це буде останній алгоритм сортування, який ми тут висвітлюємо. Однак пізніше в розділі про деревовидні структури даних ми опишемо сортування по купі, ще один ефективний метод сортування, який вимагає двійкової купи у своїй реалізації.

**Instructions:**Напишіть функцію `mergeSort` яка приймає масив цілих чисел як введення і повертає масив цих цілих чисел у відсортованому порядку від найменшого до найбільшого. Хороший спосіб реалізувати це - написати одну функцію, наприклад `merge`, яка відповідає за злиття двох сортованих масивів, та інша функція, наприклад, `mergeSort`, яка відповідає за рекурсію, що виробляє одноелементні масиви, для каналу на злиття. Успіхів!

# --hints--

`mergeSort` повинен бути функцією.

```js
assert(typeof mergeSort == 'function');
```

`mergeSort` повинен повертати відсортований масив (від найменшого до найбільшого).

```js
assert(
  isSorted(
    mergeSort([
      1,
      4,
      2,
      8,
      345,
      123,
      43,
      32,
      5643,
      63,
      123,
      43,
      2,
      55,
      1,
      234,
      92
    ])
  )
);
```

`mergeSort([1,4,2,8,345,123,43,32,5643,63,123,43,2,55,1,234,92])` повинен повернути масив без змін, крім послідовності.

```js
assert.sameMembers(
  mergeSort([
    1,
    4,
    2,
    8,
    345,
    123,
    43,
    32,
    5643,
    63,
    123,
    43,
    2,
    55,
    1,
    234,
    92
  ]),
  [1, 4, 2, 8, 345, 123, 43, 32, 5643, 63, 123, 43, 2, 55, 1, 234, 92]
);
```

`mergeSort` не повинен використовувати вбудований `.sort()` метод.

```js
assert(isBuiltInSortUsed());
```

# --seed--

## --after-user-code--

```js
function isSorted(a){
  for(let i = 0; i < a.length - 1; i++)
    if(a[i] > a[i + 1])
      return false;
  return true;
}

function isBuiltInSortUsed(){
  let sortUsed = false;
  Array.prototype.sort = () => sortUsed = true;
  mergeSort([0, 1]);
  return !sortUsed;
}
```

## --seed-contents--

```js
function mergeSort(array) {
  // Only change code below this line
  return array;
  // Only change code above this line
}
```

# --solutions--

```js
function mergeSort(array) {
  if (array.length === 1) {
    return array;
  } else {
    const splitIndex = Math.floor(array.length / 2);
    return merge(
      mergeSort(array.slice(0, splitIndex)),
      mergeSort(array.slice(splitIndex))
    );
  }

  // Merge two sorted arrays
  function merge(array1, array2) {
    let merged = [];
    while (array1.length && array2.length) {
      if (array1[0] < array2[0]) {
        merged.push(array1.shift());
      } else if (array1[0] > array2[0]) {
        merged.push(array2.shift());
      } else {
        merged.push(array1.shift(), array2.shift());
      }
    }

    // After looping ends, one array is empty, and other array contains only
    // values greater than all values in `merged`
    return [...merged, ...array1, ...array2];
  }
}

mergeSort([1, 4, 2, 8, 345, 123, 43, 32, 5643, 63, 123, 43, 2, 55, 1, 234, 92]);
```
